/*  Project: Open Source Gmini Tools
    file: ADC.h, abstraction of Analog to digital converter

    Copyright (C) 2004  The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Emulator.h"
#include <assert.h>

ADC::ADC( Emulator * gm ):gmini( gm ) {
   releaseAll(  );
};

void ADC::interact( u8 & control, u16 & data ) {
   if( control & 0x01 ) {
      // ADC enabled

      int chan = ( control >> 4 ) & 0x07;
      // finish conversion immediately: (should take a few cycles)
      control &= 0xFE;          // disable
      control |= 0x08;          // end of conversion
      if( TRACE_ADC )
         gmini->tracer.log( "ADC read channel %d=%x", chan, channels[chan] );
      data = channels[chan] << 6;   // pad left
   }
}

void ADC::releaseAll(  ) {
   // set all channels to the max voltage
   for( int i = 0; i < NUM_CHANNELS; i++ ) {
      channels[i] = 0x3FF;
   }
}
