/*  Project: Open Source Gmini Tools
    file: MMIO.h, header file for MMIO.cpp

    Copyright (C) 2004  The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef MMIOH
#define MMIOH

#include "types.h"

class Emulator;

class MMIO {
 private:
   Emulator * gmini;
 public:
   static const char mapped[400][10];
   enum mmregs {
      OSCCON = 0x001,
      WTCON = 0x002,
      BTCON = 0x004,
      BTCNT = 0x005,
      WDTEN = 0x006,
      WDTCON = 0x007,
      TACON = 0x008,
      TAPRE = 0x009,
      TADATAH = 0x00a,
      TADATAL = 0x00b,
      TACNTH = 0x00c,
      TACNTL = 0x00d,
      TBCON = 0x010,
      TBPRE = 0x011,
      TBDATAH = 0x012,
      TBDATAL = 0x013,
      TBCNTH = 0x014,
      TBCNTL = 0x015,
      TCCON = 0x018,
      TCPRE = 0x019,
      TCDATAH = 0x01a,
      TCDATAL = 0x01b,
      TCCNTH = 0x01c,
      TCCNTL = 0x01d,
      IRQ0H = 0x020,
      IRQ0L = 0x021,
      IMR0H = 0x022,
      IMR0L = 0x023,
      IPR0H = 0x024,
      IPR0L = 0x025,
      IIR0H = 0x026,
      IRQ1H = 0x028,
      IRQ1L = 0x029,
      IMR1H = 0x02a,
      IMR1L = 0x02b,
      IPR1H = 0x02c,
      IPR1L = 0x02d,
      P0 = 0x030,
      P1 = 0x031,
      P2 = 0x032,
      P3 = 0x033,
      P4 = 0x034,
      P5 = 0x035,
      P6 = 0x036,
      P7 = 0x037,
      P8 = 0x038,
      P9 = 0x039,
      P10 = 0x03a,
      P0CON = 0x040,
      P1CON = 0x041,
      P2CONH = 0x042,
      P2CONL = 0x043,
      P3CONH = 0x044,
      P3CONL = 0x045,
      P3PUR = 0x046,
      P5CON = 0x048,
      P5PUR = 0x049,
      P5INTMODH = 0x04a,
      P5INTMODL = 0x04b,
      P5INTCON = 0x04c,
      P4CON = 0x050,
      P4INTCON = 0x051,
      P4INTMOD = 0x052,
      P6CON = 0x053,
      P7CON = 0x054,
      P8CON = 0x055,
      P9CON = 0x056,
      P10CON = 0x057,
      SMCON = 0x058,
      ECCNT = 0x059,
      ECCH = 0x05a,
      ECCL = 0x05b,
      ECCX = 0x05c,
      ECCCLR = 0x05d,
      ECCRSTH = 0x05e,
      ECCRSTL = 0x05f,
      PPDATA = 0x060,
      PPCDATA = 0x061,
      PPSCON = 0x062,
      PPSTAT = 0x063,
      PPCONH = 0x064,
      PPCONL = 0x065,
      PPINTCONH = 0x066,
      PPINTCONL = 0x067,
      PPINTPNDH = 0x068,
      PPINTPNDL = 0x069,
      PPACKD = 0x06a,
      SIOCON = 0x070,
      SIOPS = 0x071,
      SIODATA = 0x072,
      ADDATAH = 0x074,
      ADDATAL = 0x075,
      ADCON = 0x076,
      FUNADDR = 0x080,
      PWRMAN = 0x081,
      FRAMELO = 0x082,
      FRAMEHI = 0x083,
      INTREG = 0x084,
      INTENA = 0x085,
      EPINDEX = 0x086,
      EPDIR = 0x089,
      INCSR = 0x08a,
      OUTCSR = 0x08b,
      INMAXP = 0x08c,
      OUTMAXP = 0x08d,
      WRTCNTLO = 0x08e,
      WRTCNTHI = 0x08f,
      EP0FIFO = 0x090,
      EP1FIFO = 0x091,
      EP2FIFO = 0x092,
      EP3FIFO = 0x093,
      IISCON0 = 0x0a0,
      IISMODE0 = 0x0a1,
      IISPTR0 = 0x0a2,
      IISCON1 = 0x0a4,
      IISMODE1 = 0x0a5,
      IISPTR1 = 0x0a6,
      PLL0DATAH = 0x0a8,
      PLL0DATAL = 0x0a9,
      PLL0CON = 0x0aa,
      PLL1DATAH = 0x0ac,
      PLL1DATAL = 0x0ad,
      PLL1CON = 0x0ae,
      LCON = 0x0b0,
      UCON = 0x0b1,
      USSR = 0x0b2,
      TBR = 0x0b3,
      RBR = 0x0b4,
      UBRDR = 0x0b5,
      UPEND = 0x0b6,
      IICCON = 0x0b8,
      IICSR = 0x0b9,
      IICDATA = 0x0ba,
      IICADDR = 0x0bb,
      IICCNT = 0x0bd,
      IISBUF = 0x0c0,
      IISBUF_LAST = 0x0FF,
      IDSC0H = 0x100,
      IDSC0L = 0x101,
      IDSC1H = 0x102,
      IDSC1L = 0x103,
      IDSC2H = 0x104,
      IDSC2L = 0x105,
      IDSC3H = 0x106,
      IDSC3L = 0x107,
      RANCON = 0x108,
      LFSR8 = 0x109,
      LFSR16H = 0x10a,
      LFSR16L = 0x10b,
      INTMODE = 0x10c,
      MIUSCFG = 0x110,
      MIUDCMD = 0x112,
      MIUDCFG = 0x113,
      MIUDCNTH = 0x114,
      MIUDCNTL = 0x115,
      DDMACOM = 0x120,
      DDMACFG = 0x121,
      DDMAIADRX = 0x123,
      DDMAIADRH = 0x124,
      DDMAIADRL = 0x125,
      DDMAEADRX = 0x127,
      DDMAEADRH = 0x128,
      DDMAEADRL = 0x129,
      DDMANUMH = 0x12a,
      DDMANUML = 0x12b,
      DDMACNTH = 0x12c,
      DDMACNTL = 0x12d,
      DDMANUMX = 0x12e,
      DDMACNTX = 0x12f,
      YDMACOM = 0x130,
      YDMACFG = 0x131,
      YDMAIADRH = 0x132,
      YDMAIADRL = 0x133,
      YDMAEADRX = 0x135,
      YDMAEADRH = 0x136,
      YDMAEADRL = 0x137,
      YDMANUMH = 0x138,
      YDMANUML = 0x139,
      YDMACNTH = 0x13a,
      YDMACNTL = 0x13b,
      CACHECON = 0x140,
      ICBASEH = 0x141,
      ICBASEL = 0x142,
      DCBASEH = 0x143,
      DCBASEL = 0x144,
      LCD_BASE = 0x150,
      LCNTR0H = 0x150,
      LCNTR0L = 0x151,
      LCNTR1H = 0x152,
      LCNTR1L = 0x153,
      LCNTR2H = 0x154,
      LCNTR2L = 0x155,
      LCNTR3H = 0x156,
      LCNTR3L = 0x157,
      LCNTR4H = 0x158,
      LCNTR4L = 0x159,
      LCNTR5H = 0x15a,
      LCNTR5L = 0x15b,
      LDBAR1H = 0x15d,
      LDBAR1M = 0x15e,
      LDBAR1L = 0x15f,
      LDBAR2H = 0x161,
      LDBAR2M = 0x162,
      LDBAR2L = 0x163,
      LDCAR1H = 0x165,
      LDCAR1M = 0x166,
      LDCAR1L = 0x167,
      LDCAR2H = 0x169,
      LDCAR2M = 0x16a,
      LDCAR2L = 0x16b,
      LSR = 0x16d,
      LRLUTRHH = 0x16e,
      LRLUTRHL = 0x16f,
      LRLUTRLH = 0x170,
      LRLUTRLL = 0x171,
      LGLUTRHH = 0x172,
      LGLUTRHL = 0x173,
      LGLUTRLH = 0x174,
      LGLUTRLL = 0x175,
      LBLUTRH = 0x176,
      LBLUTRL = 0x177,
      LDP1_7 = 0x178,
      LDP3_7 = 0x179,
      LDP4_7 = 0x17a,
      LDP5_7 = 0x17b,
      LDP6_7 = 0x17c,
      LDP1_5 = 0x17e,
      LDP2_5 = 0x17f,
      LDP3_5 = 0x180,
      LDP4_5 = 0x181,
      LDP1_4 = 0x182,
      LDP3_4 = 0x183,
      LDP1_3 = 0x184,
      MMIO_END
   };


 private:

   u8 io[MMIO_END];
#ifdef __GMEMU_DEBUG__
   int ioaccess[MMIO_END][2];
#endif
   enum FlashState {FS_1, FS_2, FS_COMMAND, FS_SOFTWARE_ID,
		    FS_ERASE1, FS_ERASE2, FS_ERASE_COMMAND, 
		    FS_WORD_PROGRAM};
   FlashState flashState;
 public:
   MMIO( Emulator * gm );
   u8 read8( u22 address );
   u16 read16( u22 address );
   bool write8( u22 address, u8 value );
   bool write16( u22 address, u16 value );
   u8 rawread8( u22 address );
   u16 rawread16( u22 address );
   void rawwrite8( u22 address, u8 value );
   void rawwrite16( u22 address, u16 value );
   void dump(  );
   void saveState(  );
   void restoreState(  );
 private:
   bool dump8( const char *datafile, u8 byte );
   bool dump16( const char *datafile, u16 byte );
   void ddmaStart(  );
   void ydmaStart(  );
   void p3Interact( bool readLines );

};
#endif
