/*  Project: Open Source Gmini Tools
    file: MMU.h, header file for MMU.cpp

    Copyright (C) 2004  The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef MMUH
#define MMUH
#include <vector>
#include "MMIO.h"

#define NB_LINES_DUMP_SCREEN 3

class MMU {
 public:
   enum { FLASHSIZE = 0x100000 };
   enum { MEMSIZE = 0x3f0000 };
   enum { FIQMEM = 0x3FC000 };
   enum { FIQSIZE = 0x4000 };
   enum { IOBASE = 0x3f0000 };

 private:

   enum { RAMBASE = 0x000000 };
   enum { RAMSIZE = 0x3F0000 };
   enum { RAMPAGE = 0x200000 };

   enum { CODEBASE = 0x000000 };
   enum { DATABASE = 0x080000 };
   enum { M1BASE = 0x200000 };
   enum { M2BASE = 0x210000 };
   enum { M3BASE = 0x220000 };
   enum { M4BASE = 0x230000 };
   enum { ICBASE = 0x3fc400 };

   enum { CODESIZE = 0x80000 };
   enum { DATASIZE = 0x80000 };
   enum { M1SIZE = 0x8000 };
   enum { M2SIZE = 0x8000 };
   enum { M3SIZE = 0x4000 };
   enum { M4SIZE = 0x4000 };
   enum { IOSIZE = 0x190 };
   enum { ICSIZE = 0x3fc0 };

   u8 *flash;
   u8 *fiq;
   u8 *ram;
   Emulator *gmini;

   u8 *mem( u22 address );
   u22 base( u22 address );

   static const char stateMEMfile[14];
   static const char stateRAMfile[14];
   static const char stateFIQfile[14];

   bool loadmem( const char *datafile );
   bool loadram( const char *datafile );
   bool loadfiq( const char *datafile );
 public:
   enum access { READ16, READ8, WRITE16, WRITE8, CODE32,
      CODE16, FIRM, RJUMP, AJUMP, DATA
   };
   MMU( Emulator * gm );
   ~MMU(  );
   const u8 dread8( u22 address );
   const u16 dread16( u22 address );
   const u24 xread24( u16 address );
   void xwrite24( u16 address, u24 value );
   bool dwrite8( u22 address, u8 value );
   bool dwrite16( u22 address, u16 value );
   const u16 cread16( u22 address );
   bool cwrite16( u22 address, u16 value );
   void drawwrite32( u22 address, u32 value );
   const u32 drawread32( u22 address );
   void cmemcopy( u32 destination, u32 source, u32 size );
   void dmemcopy( u32 destination, u32 source, u32 size );
   void ydma(int dir, int blk, u16 intAddr, u22 extAddr, u16 num);

   u8 *getdmem(  );
   const u32 find( const u8 * value, u8 length );
   const u16 cread( u22 address ); // used only for retriving opcodes;
   bool cwrite( u22 address, u8 value );  // used only for loading firmware file
   const char *getcstr( char *buffer, u22 address, int maxlength );
   const char *getdstr( char *buffer, u22 address, int maxlength );
   bool load( const char *filename );
   bool initialize( const char *filename );
   void findStrings(  );
   const void dumpScreen( u22 address, int count = NB_LINES_DUMP_SCREEN * 0x10 );

   const bool dumpmem( const char *datafile );
   const bool dumpram( const char *datafile );
   const bool dumpfiq( const char *datafile );

   const void saveState(  );
   void restoreState(  );
};
#endif
