
#include "qgmemu.h"
#include <assert.h>
#include <qlayout.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qtextedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qwhatsthis.h>
#include <qtimer.h>
#include <qhbox.h>

#include "filesave.xpm"
#include "fileopen.xpm"
#include "fileprint.xpm"

#include "parsecmd.hh"
#include "gmlcd.h"
#include "gmregisters.h"
#include "Console.h"
#include <qthread.h>
#include "Emulator.h"
#include <qpainter.h>
#include <qpixmap.h>
#include <qmainwindow.h>



class ThreadEmulator : public QThread {
public:
    ThreadEmulator(Emulator *e){
        emulator = e;
    }
    virtual void run();
private:
    Emulator *emulator;
};

void ThreadEmulator::run()
{
  //emulator->cpu.run();
  startparser();
  while (1)
    parsecmd(emulator);
  stopparser();
}

// class ThreadLCD : public QThread {
// public:
//     ThreadLCD(Emulator *e, GMLCD *l){
//         emulator = e;
// 	lcd = l;
//     }
//     virtual void run();
// private:
//     Emulator *emulator;
//     GMLCD *lcd;
// };

// void ThreadLCD::run()
// {
//     u8* buf = emulator->memory.getdmem(  ) + 0x1810;
//     QRgb colorMap[2];
//     colorMap[0] = Qt::white.rgb();
//     colorMap[1] = Qt::black.rgb();
//     QImage img(buf, 128, 64, 1, colorMap, 2, QImage::BigEndian);

//     while (true)
//     {
//  	bitBlt(lcd, 0, 0, &img, 0, 0, 128, 64);
// 	sleep(1);
//     }
// }

void qgmemu::updateLcd() {
  QRgb colorMap[256];
  for (int i = 4; i < 256; i++)
    colorMap[i] = Qt::blue.red(); // invalid colors (bug in rendering)

  int w,h,depth;

  static u8 buf[160*160]; // max buffer size

  if (gmini->lcd.updated) {
    //console.output("refreshing LCD\n");
    gmini->lcd.updated = false; // the race is a short one, we don't care
    w = 128;
    h = 65; // 64 shows a line less! (bug in QT ?)
    gmini->lcd.dumpImage(buf);
    depth = 1;
  } else
  if (gmini->lcdi.enabled()) {
    h = gmini->lcdi.height();
    w = gmini->lcdi.width();
    depth = gmini->lcdi.bpp();
    // fixme: move this to lcdi.dumpImage as above.
    long framebufferAddr = gmini->lcdi.frameBuffer();
    u8* frameBuffer = ((u8*) gmini->memory.getdmem()) + framebufferAddr;
    assert (depth == 4);
    for (int i = 0; i < w*h/2; i++) {
      u8 x = frameBuffer[i];
      buf[i*2] = x >> 4;
      buf[i*2+1] = x & 0x0F;
    }
  } else
    return;

  assert (h <= 160);
  assert (w <= 160);

  bool backlight = gmini->io.rawread8(MMIO::P1) & 0x10;

  int levels = 1 << depth;
  for (int i = 0; i<levels; i++) {
    QColor c(230, backlight ? 40 : 0, (levels-i-1)*250/(levels-1), QColor::Hsv);
    colorMap[i] = c.rgb();
  }

  QImage img(buf, w, h, 8, colorMap, 256, QImage::IgnoreEndian);
  bitBlt(lcd, 0, 0, &img, 0, 0, w, h);
}

void qgmemu::pause() {
 gmini->cpu.sigint(  );
}

bool cmdexit; // kludge

qgmemu::qgmemu(int argc, const char* argv[])
    : QMainWindow( 0, "qgmemu", WDestructiveClose )
{
    ThreadEmulator *thread_emulator;
    //    ThreadLCD *thread_lcd;

    printer = new QPrinter;
    QPixmap openIcon, saveIcon, printIcon;

    QToolBar * fileTools = new QToolBar( this, "file operations" );
    fileTools->setLabel( tr("File Operations") );

    QPushButton * pause
	= new QPushButton( "||", fileTools, "pause" );
    connect(pause, SIGNAL(clicked()), this, SLOT(pause()));

    openIcon = QPixmap( fileopen );
    QToolButton * fileOpen
	= new QToolButton( openIcon, tr("Open File"), QString::null,
			   this, SLOT(choose()), fileTools, "open file" );

    saveIcon = QPixmap( filesave );
    QToolButton * fileSave
	= new QToolButton( saveIcon, tr("Save File"), QString::null,
			   this, SLOT(save()), fileTools, "save file" );

    printIcon = QPixmap( fileprint );
    QToolButton * filePrint
	= new QToolButton( printIcon, tr("Print File"), QString::null,
			   this, SLOT(print()), fileTools, "print file" );


    (void)QWhatsThis::whatsThisButton( fileTools );

    QString fileOpenText = tr("<p><img source=\"fileopen\"> "
	         "Click this button to open a <em>new file</em>. <br>"
                 "You can also select the <b>Open</b> command "
                 "from the <b>File</b> menu.</p>");

    QWhatsThis::add( fileOpen, fileOpenText );

    QMimeSourceFactory::defaultFactory()->setPixmap( "fileopen", openIcon );

    QString fileSaveText = tr("<p>Click this button to save the file you "
                 "are editing. You will be prompted for a file name.\n"
                 "You can also select the <b>Save</b> command "
                 "from the <b>File</b> menu.</p>");

    QWhatsThis::add( fileSave, fileSaveText );

    QString filePrintText = tr("Click this button to print the file you "
                 "are editing.\n You can also select the Print "
                 "command from the File menu.");

    QWhatsThis::add( filePrint, filePrintText );


    QPopupMenu * file = new QPopupMenu( this );
    menuBar()->insertItem( tr("&File"), file );



    int id;
    id = file->insertItem( openIcon, tr("&Open..."),
			   this, SLOT(choose()), CTRL+Key_O );
    file->setWhatsThis( id, fileOpenText );

    id = file->insertItem( saveIcon, tr("&Save"),
			   this, SLOT(save()), CTRL+Key_S );
    file->setWhatsThis( id, fileSaveText );

    id = file->insertItem( tr("Save &As..."), this, SLOT(saveAs()) );
    file->setWhatsThis( id, fileSaveText );

    file->insertSeparator();

    id = file->insertItem( printIcon, tr("&Print..."),
			   this, SLOT(print()), CTRL+Key_P );
    file->setWhatsThis( id, filePrintText );

    file->insertSeparator();

    file->insertItem( tr("&Close"), this, SLOT(close()), CTRL+Key_W );

    file->insertItem( tr("&Quit"), qApp, SLOT( closeAllWindows() ), CTRL+Key_Q );

    menuBar()->insertSeparator();

    QPopupMenu * help = new QPopupMenu( this );
    menuBar()->insertItem( tr("&Help"), help );

    help->insertItem( tr("&About"), this, SLOT(about()), Key_F1 );
    help->insertItem( tr("About &Qt"), this, SLOT(aboutQt()) );
    help->insertSeparator();
    help->insertItem( tr("What's &This"), this, SLOT(whatsThis()), SHIFT+Key_F1 );

    // Editor
    e = new QTextEdit( this, "editor" );
    e->setFocus();
    setCentralWidget( e );
    statusBar()->message( tr("Ready"), 2000 );
    resize( 450, 600 );

    //LCD
    QToolBar * tb_lcd = new QToolBar( this, "TBLCD" );
    tb_lcd->setLabel( tr("LCD") );
    lcd = new GMLCD(tb_lcd, "LCD");
    QMainWindow::moveDockWindow(tb_lcd, Qt::DockLeft);
   
    /* 
       //Regs
    QToolBar * tb_reg = new QToolBar( this, "TBREG" );
    tb_reg->setLabel( tr("Registers") );
    reg = new GMRegisters(tb_reg, "REG");
    QMainWindow::moveDockWindow(tb_reg, Qt::DockLeft);
   

    //console
    QToolBar * tb_con = new QToolBar( this, "TBCON" );
    tb_con->setLabel( tr("Console") );
    console.createWidget(tb_con);
    QMainWindow::moveDockWindow(tb_con, Qt::DockBottom);
    */

    gmini = new Emulator(argc, argv);
    thread_emulator = new ThreadEmulator(gmini);
    thread_emulator->start(QThread::LowPriority);

    QTimer* lcdTrigger = new QTimer();
    
    lcdTrigger->start(333, false);
    connect( lcdTrigger, SIGNAL(timeout()), this, SLOT(updateLcd()) );
    QTimer* trigger = new QTimer(); 
    
    trigger->start(500, false);


    const int watchedIoNum = 5;
    int watchedIo [watchedIoNum] = {MMIO::P7, MMIO::IMR0H, MMIO::IMR0L, 
				    MMIO::IMR1H, MMIO::IMR1L};
    // IO Ports
    for (int i =0; i < watchedIoNum; i++) {
      
      QToolBar * tb_wa = new QToolBar( this, "WATCH" );
       IOWatch* tst = new IOWatch(gmini, watchedIo[i], tb_wa);
       connect( trigger, SIGNAL(timeout()), tst, SLOT(refresh()) );
       QMainWindow::moveDockWindow(tb_wa, Qt::DockRight);
    }



     // ADC channels
     for (int i = 0; i < 8; i++) {
      char buffer[10];
      sprintf(buffer, "ADC%d", i);
      QToolBar * tb_wa = new QToolBar( this, "WATCH" );
      AddrWatch* tst = new AddrWatch(buffer, &(gmini->adc.channels[i]), tb_wa);
      connect( trigger, SIGNAL(timeout()), tst, SLOT(refresh()) );

    }


    
       
    { //Keypad
      QToolBar* tb = new QToolBar( this, "Buttons" );
      QWidget* kp = new QWidget(tb, "keypad");
      QGridLayout *grid = new QGridLayout(kp, 3, 4);

      butDown = new QPushButton(kp, "DOWN");
      butUp = new QPushButton(kp, "UP");
      butLeft = new QPushButton(kp, "LEFT");
      butRight = new QPushButton(kp, "RIGHT");
      butOn = new QPushButton(kp, "ON");
      butOk = new QPushButton(kp, "OK");
      butCancel = new QPushButton(kp, "CANCEL");
      butMenu = new QPushButton(kp, "MENU");

      
      butDown->setText("v");
      butUp->setText("^");
      butLeft->setText("<");
      butRight->setText(">");
      butOn->setText("!");
      butOk->setText(">|");
      butCancel->setText("X");
      butMenu->setText("?");

      butDown->setFixedSize(16,16);
      butUp->setFixedSize(16,16);
      butLeft->setFixedSize(16,16);
      butRight->setFixedSize(16,16);
      butOk->setFixedSize(16,16);
      butOn->setFixedSize(16,16);
      butCancel->setFixedSize(16,16);
      butMenu->setFixedSize(16,16);

      grid->addWidget(butDown,  2, 1);   
      grid->addWidget(butUp,    0, 1);   
      grid->addWidget(butLeft,  1, 0);   
      grid->addWidget(butRight, 1, 2);   
      grid->addWidget(butOk,    1, 1);   
      grid->addWidget(butOn,    3, 1);   
      grid->addWidget(butCancel,3, 2);   
      grid->addWidget(butMenu,  3, 0);

      connect(butUp, SIGNAL(pressed()), this, SLOT(buttonPressOrRelease()));
      connect(butUp, SIGNAL(released()), this, SLOT(buttonPressOrRelease()));

      connect(butDown, SIGNAL(pressed()), this, SLOT(buttonPressOrRelease()));
      connect(butDown, SIGNAL(released()), this, SLOT(buttonPressOrRelease()));

      connect(butLeft, SIGNAL(pressed()), this, SLOT(buttonPressOrRelease()));
      connect(butLeft, SIGNAL(released()), this, SLOT(buttonPressOrRelease()));

      connect(butRight, SIGNAL(pressed()), this, SLOT(buttonPressOrRelease()));
      connect(butRight, SIGNAL(released()), this, SLOT(buttonPressOrRelease()));

      connect(butOk, SIGNAL(pressed()), this, SLOT(buttonPressOrRelease()));
      connect(butOk, SIGNAL(released()), this, SLOT(buttonPressOrRelease()));

      connect(butOn, SIGNAL(pressed()), this, SLOT(buttonPressOrRelease()));
      connect(butOn, SIGNAL(released()), this, SLOT(buttonPressOrRelease()));

      connect(butCancel, SIGNAL(pressed()), this, SLOT(buttonPressOrRelease()));
      connect(butCancel, SIGNAL(released()), this, SLOT(buttonPressOrRelease()));

      connect(butMenu, SIGNAL(pressed()), this, SLOT(buttonPressOrRelease()));
      connect(butMenu, SIGNAL(released()), this, SLOT(buttonPressOrRelease()));

      QMainWindow::moveDockWindow(tb, Qt::DockLeft);
    }
    
}



void qgmemu::buttonPressOrRelease() {
//   gmini->adc.channels[6] = 0x200;
//   gmini->adc.channels[7] = 0x200;

  gmini->adc.channels[6] = 0x3FF;
  gmini->adc.channels[7] = 0x3FF;

  char x = gmini->io.rawread8(MMIO::P7);
  x &= ~1;
  if (butOn->isDown()) {
    x |= 1;
  }
  gmini->io.rawwrite8(MMIO::P7, x);

  if (butOk->isDown()) {
    gmini->adc.channels[6] = 0x268;
  }

  if (butUp->isDown()) {
    gmini->adc.channels[7] = 0x300;
  }
  if (butLeft->isDown()) {
    gmini->adc.channels[7] = 0x080;
  }
  if (butDown->isDown()) {
    gmini->adc.channels[7] = 0x180;
  }
  if (butRight->isDown()) {
    gmini->adc.channels[7] = 0x200;
  }
  if (butCancel->isDown()) {
    gmini->adc.channels[6] = 0x200;
  }
  if (butMenu->isDown()) {
    gmini->adc.channels[6] = 0x2a8;
  }


}


#define BITS 10

AddrWatch::AddrWatch (QString name, u16* watched, QWidget* parent): QWidget(parent) {
  this->watched = watched;
  
  QGridLayout *grid = new QGridLayout(this, BITS+1, 1);
  QLabel* lab = new QLabel(this, "WATCHNAME");
  lab->setText(name);
  grid->addWidget(lab, 0, 0);
  char buffer[4];
  for (int i = 0; i < BITS; i++) {
    buttons[i] = new QPushButton(this, "BIT");
    buttons[i]->setToggleButton(true);
    sprintf(buffer, "%d", i);
    buttons[i]->setText(buffer);
    buttons[i]->setFixedSize(16,16);    
    grid->addWidget(buttons[i], 0, BITS-i);
    connect(buttons[i], SIGNAL(toggled(bool)), this, SLOT(modify()));
  }

}

void AddrWatch::modify() {
  u32 res = 0;
  for (int i = 0; i < BITS; i++) {
    res = res >> 1;
    if (buttons[i]->isOn()) 
      res |= (1<<(BITS-1));
  }
  *watched = res;
}
void AddrWatch::refresh() {
  u32 res = *watched;
  for (int i = 0; i < BITS; i++) {
    buttons[i]->setOn(res&1);
    res = res >> 1;
  }
}

IOWatch::IOWatch (Emulator* gmini, int watched, QWidget* parent): QWidget(parent) {
  this->gmini = gmini;
  this->watched = watched;
  
  QGridLayout *grid = new QGridLayout(this, 9, 1);
  QLabel* lab = new QLabel(this, "WATCHNAME");
  lab->setText(MMIO::mapped[watched]);
  grid->addWidget(lab, 0, 0);
  char buffer[4];
  for (int i = 0; i < 8; i++) {
    buttons[i] = new QPushButton(this, "BIT");
    buttons[i]->setToggleButton(true);
    sprintf(buffer, "%d", i);
    buttons[i]->setText(buffer);
    buttons[i]->setFixedSize(16,16);    
    grid->addWidget(buttons[i], 0, 8-i);
    connect(buttons[i], SIGNAL(toggled(bool)), this, SLOT(modify()));
  }

}

void IOWatch::modify() {
  u8 res = 0;
  for (int i = 0; i < 8; i++) {
    res = res >> 1;
    if (buttons[i]->isOn()) 
      res |= 0x80;
  }
  gmini->io.rawwrite8(watched, res);
}
void IOWatch::refresh() {
  u8 res = gmini->io.rawread8(watched);
  for (int i = 0; i < 8; i++) {
    buttons[i]->setOn(res&1);
    res = res >> 1;
  }
}



qgmemu::~qgmemu()
{
    delete printer;
    delete gmini;
}


void qgmemu::choose()
{
    QString fn = QFileDialog::getOpenFileName( QString::null, QString::null,
					       this);
    if ( !fn.isEmpty() )
	load( fn );
    else
	statusBar()->message( tr("Loading aborted"), 2000 );
}


void qgmemu::load( const QString &fileName )
{
    QFile f( fileName );
    if ( !f.open( IO_ReadOnly ) )
	return;

    QTextStream ts( &f );
    e->setText( ts.read() );
    e->setModified( FALSE );
    setCaption( fileName );
    statusBar()->message( tr("Loaded document %1").arg(fileName), 2000 );
}


void qgmemu::save()
{
    if ( filename.isEmpty() ) {
	saveAs();
	return;
    }

    QString text = e->text();
    QFile f( filename );
    if ( !f.open( IO_WriteOnly ) ) {
	statusBar()->message( tr("Could not write to %1").arg(filename),
			      2000 );
	return;
    }

    QTextStream t( &f );
    t << text;
    f.close();

    e->setModified( FALSE );

    setCaption( filename );

    statusBar()->message( tr( "File %1 saved" ).arg( filename ), 2000 );
}


void qgmemu::saveAs()
{
    QString fn = QFileDialog::getSaveFileName( QString::null, QString::null,
					       this );
    if ( !fn.isEmpty() ) {
	filename = fn;
	save();
    } else {
	statusBar()->message( tr("Saving aborted"), 2000 );
    }
}


void qgmemu::print()
{
    // ###### Rewrite to use QSimpleRichText to print here as well
    const int Margin = 10;
    int pageNo = 1;

    if ( printer->setup(this) ) {		// printer dialog
	statusBar()->message( tr("Printing...") );
	QPainter p;
	if( !p.begin( printer ) )               // paint on printer
	    return;

	p.setFont( e->font() );
	int yPos	= 0;			// y-position for each line
	QFontMetrics fm = p.fontMetrics();
	QPaintDeviceMetrics metrics( printer ); // need width/height
						// of printer surface
	for( int i = 0 ; i < e->lines() ; i++ ) {
	    if ( Margin + yPos > metrics.height() - Margin ) {
		QString msg( "Printing (page " );
		msg += QString::number( ++pageNo );
		msg += ")...";
		statusBar()->message( msg );
		printer->newPage();		// no more room on this page
		yPos = 0;			// back to top of page
	    }
	    p.drawText( Margin, Margin + yPos,
			metrics.width(), fm.lineSpacing(),
			ExpandTabs | DontClip,
			e->text( i ) );
	    yPos = yPos + fm.lineSpacing();
	}
	p.end();				// send job to printer
	statusBar()->message( tr("Printing completed"), 2000 );
    } else {
	statusBar()->message( tr("Printing aborted"), 2000 );
    }
}

void qgmemu::closeEvent( QCloseEvent* ce )
{
    if ( !e->isModified() ) {
	ce->accept();
	return;
    }

    switch( QMessageBox::information( this, tr("Qt Application Example"),
				      tr("Do you want to save the changes"
				      " to the document?"),
				      tr("Yes"), tr("No"), tr("Cancel"),
				      0, 1 ) ) {
    case 0:
	save();
	ce->accept();
	break;
    case 1:
	ce->accept();
	break;
    case 2:
    default: // just for sanity
	ce->ignore();
	break;
    }
}


void qgmemu::about()
{
    QMessageBox::about( this, tr("Qt Application Example"),
			tr("This example demonstrates simple use of "
			"QMainWindow,\nQMenuBar and QToolBar."));
}


void qgmemu::aboutQt()
{
    QMessageBox::aboutQt( this, tr("Qt Application Example") );
}
