%{

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "parsecmd.h"

#define YY_INPUT(buf,result,max_size) (result=yy_input(buf,max_size))

static char* current_command_ptr=NULL;

int yy_set_input(char* newInput) {
   if (current_command_ptr!=NULL)
      YY_FLUSH_BUFFER;
   current_command_ptr = newInput;
}

int yy_input(char* buf, int max_size) {
  if (* current_command_ptr == 0)
     return YY_NULL;
  else {
     int n = strlen(current_command_ptr)+1;
     if (n > max_size) n = max_size;
     memcpy(buf, current_command_ptr, n);
     current_command_ptr += n;
     return n;
  }
     
}

%}

bpm		bpm
bpr		bpr
bpx		bpx
bt		bt
clear		c(l(e(a(r)?)?)?)?
delete		de(l(e(t(e)?)?)?)?
disasm          di(sasm)?
dump		du(m(p)?)?
flash		f(l(a(s(h)?)?)?)?
go		g(o)?
help		(h(e(l(p)?)?)?|\?)
jog		j(o(g)?)?
lcd             lcd
list		li(s(t)?)?
load		lo(a(d)?)?
mmio		m(m(i(o)?)?)?
next            n(e(xt?)?)?
ops		o(p(s)?)?
quit		q(u(i(t)?)?)?
ram		ra(m)?
registers	reg(i(s(t(e(r(s)?)?)?)?)?|s)?
restore		res(t(o(r(e)?)?)?)?
run		ru(n)?
r2r		r2(r)?
save		sa(v(e)?)?
set		se(t)?
step		st(e(p)?)?
trace           tr(a(ce?)?)?
walk		wa(l(k)?)?
decimal		[0-9]+
hexnum		0x[0-9a-fA-F]+
read		R
write		W
execute		X
filename	\"[0-9a-zA-Z_\-]+\.[0-9a-zA-Z_\-]+\"
white		[ \t]+
cr		[\n]
ident           [a-zA-Z_][0-9a-zA-Z_]*

%%

{bpm}		{ return(_BPM); }
{bpr}		{ return(_BPR); }
{bpx}		{ return(_BPX); }
{bt}		{ return(_BT); }
{clear}		{ return(_CLEAR); }
{delete}	{ return(_DELETE); }
{disasm}	{ return(_DISASM); }
{dump}		{ return(_DUMP); }
{flash}		{ return(_FLASH); }
{go}		{ return(_GO); }
{help}		{ return(_HELP); }
{jog}		{ return(_JOG); }
{lcd}           { return(_LCD); }
{list}		{ return(_LIST); }
{load}		{ return(_LOAD); }
{mmio}		{ return(_MMIO); }
{next}          { return(_NEXT); }
{ops}		{ return(_OPS); }
{quit}		{ return(_QUIT); }
{read}		{ return(_R); }
r{decimal}	{ yylval.number = atoi(yytext+1); return(_RREG); }
{ram}		{ return(_RAM); }
{registers}	{ return(_REGS); }
{restore}	{ return(_RESTORE); }
{run}		{ return(_RUN); }
{r2r}		{ return(_RUN2RET); }
{save}		{ return(_SAVE); }
{set}		{ return(_SET); }
{step}		{ return(_STEP); }
{trace}         { return(_TRACE); }
{walk}		{ return(_WALK); }
{decimal}	{ yylval.address = atoi(yytext); return(_NUMBER); }
{hexnum}	{ yylval.address = strtol(yytext+2, NULL, 16); return(_ADDRESS); }
{filename}	{ yylval.string = strdup(yytext); return(_FILENAME); }
{write}		{ return(_W); }
{execute}	{ return(_X); }
{ident}         { yylval.string = yytext; return(_IDENTIFIER); }
{white}		{ /* We ignore white characters */ }
