
 /* Project: Open Source Gmini Tools file: trace.cpp, Logs events

    Copyright (C) 2004 The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
    later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
    details.

    You should have received a copy of the GNU General Public License along with this program; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

#include <stdarg.h>
#include <stdio.h>
#include <assert.h>
#include "Emulator.h"

#define TRACE_DUMP_STACK (false)
#define TRACE_TO_CONSOLE (true)
#define TRACE_DUMP_OP (false)

Tracer::Tracer( Emulator * gm ):
  gmini( gm ),
  file( "tracer.log", ios::out | ios::trunc ) {
  enabled = true;
};


void Tracer::traceStep(  ) {
   if( events.size(  ) > 0 ) {
      if( TRACE_DUMP_STACK ) {
         file << "~~~~~~~~~~" << endl;
         gmini->cpu.dumpBacktrace( file );
      }
      if( TRACE_DUMP_OP )
         gmini->cpu.dumpops( file );
      for( unsigned i = 0; i < events.size(  ); i++ ) {
         file << events[i] << endl;
      }
      events.clear(  );
   }
}

void Tracer::log( const char *fmt, ... ) {
  if (enabled) {
    static char event[1024];
    va_list ap;
    va_start( ap, fmt );
    vsnprintf( event, 1023, fmt, ap );
    va_end( ap );
    events.push_back( event );
    if (TRACE_TO_CONSOLE)
      console.output("%s\n", event);
  }
}


void Tracer::logBuffer( const u8* buffer, int size ) {
  const int maxSize = 2048;
  if (enabled) {
    assert(size < maxSize/2);
    static char text[maxSize * 2+1];
    char* ptr = text;
    for (int i = 0; i < size; i++) {
      if (i > 0 && i%32 == 0)
	ptr += sprintf(ptr, "\n");
      ptr += sprintf(ptr, "%02x", buffer[i]);
    }
    events.push_back( text );
    if (TRACE_TO_CONSOLE)
      console.output("%s\n", text);
  }
}
