/*  Project: Open Source Gmini Tools
    file: macargs.cpp, global data representing MAC2424 instruction arguments

    Copyright (C) 2004  The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "macargs.h"

char rps[8][7] = { "RP0+S0", "RP1+S0", "RP2+S0", "RP3+S0",
   "RP0+S1", "RP1+S1", "RP2+S1", "RP3+S1"
};
char rpd[8][7] = { "RP0+D0", "RP1+D0", "RP2+D0", "RP3+D0",
   "RP0+D1", "RP1+D1", "RP2+D1", "RP3+D1"
};
char rp01s[4][7] = { "RP0+S0", "RP1+S0", "RP0+S1", "RP1+S1" };
char rp3s[2][7] = { "RP3+S0", "RP3+S1" };
char mg1[8][5] = { "Y0", "Y1", "X0", "X1",
   "MA0H", "MA0L", "MA1H", "MA1L"
};
char mg2[8][5] = { "RP0", "RP1", "RP2", "RP3",
   "RPD0", "RPD1", "MC0", "MC1"
};
char sdi[4][4] = { "SD0", "SD1", "SD2", "SD3" };
char Ai[2][2] = { "A", "B" };
char Ci[2][2] = { "C", "D" };
char An[4][2] = { "A", "B", "C", "D" };
char rpui[16][6] = { "RP0", "RP1", "RP2", "RP3",
   "MC0_0", "MC1_0", "MC0_1", "MC1_1",
   "SD0_0", "SD1_0", "SD2_0", "SD3_0",
   "SD0_1", "SD1_1", "SD2_1", "SD2_1"
};
char mga[4][4] = { "MA0", "MA1", "A", "B" };
char mgx[4][3] = { "Y0", "Y1", "X0", "X1" };
char mg[32][6] = { "MA0H", "MA0L", "MA1H", "MA1L",
   "MA0SR", "MA0SL", "MA1SR", "MA1SL",
   "RP0", "RP1", "RP2", "RP3",
   "RPD0", "RPD1", "MC0", "MC1",
   "SD0", "SD1", "SD2", "SD3",
   "SA", "SI", "SG", "SR",
   "PH", "PL", "MA0RN", "MA1RN",
   "MSR0", "MSR1", "MSR2", "PRN"
};
char mci[2][4] = { "MC0", "MC1" };
char srg[4][3] = { "SA", "SI", "SG", "SR" };
char asr[4][3] = { "A", "B", "SI", "SR" };
char asa[4][3] = { "A", "B", "SA", "" };
char bs[16][5] = { "BK0", "BK1", "BK2", "BK3",
   "ME0", "ME1", "ME2", "ME3",
   "OPM", "OPMA", "OP", "USM",
   "MV", "XSD", "PSH1", "NQ"
};
char ereg[12][3] = { "AH", "BH", "CH", "DH",
   "A", "B", "C", "D",
   "SA", "SI", "SG", "SR"
};
char ns[4][3] = { "S0", "S1", "D0", "D1" };
char emod0[4][5] = { "ELD", "EADD", "ESUB", "ECP" };
char Pi[2][3] = { "PH", "PL" };
char cct[16][4] = { "Z  ", "NZ ", "NEG", "POS",
   "C  ", "NC ", "V  ", "NV ",
   "GT ", "LE ", "VM0", "VM1",
   "VS ", "  ", "MV ", "  "
};
char emod1[2][16][8] = { {"ESRA", "ESLA", "ESRA8", "ESLA8",
                          "ESRC", "ESLC", "EINCC", "EDECC",
                          "ENEG", "EABS", "EFS16", "EFZ16",
                          "EFS8", "EFZ8", "EEXP", "EEXPC"},
{"ESRAT", "ESLAT", "ESRA8T", "ESLA8T",
 "ESRCT", "ESLCT", "EINCCT", "EDECCT",
 "ENEGT", "EABST", "EFS16T", "EFZ16T",
 "EFS8T", "EFZ8T", "EEXPT", "EEXPCT"}
};
char emod2[2][16][16] = {
   {"ESRA  MA0", "ESLA  MA0", "ERND  MA0", "ECR   MA0",
    "ESAT  MA0", "ERESR MA0", "Reserved", "Reserved",
    "ELD   MA0, MA1", "Reserved", "EADD  MA0, P", "ESUB  MA0, P",
    "EADD  MA0, PSH", "ESUB  MA0, PSH", "EDIVQ MA0, P", "Reserved"},
   {"ESRA  MA1", "ESLA  MA1", "ERND  MA1", "ECR   MA1",
    "ESAT  MA1", "ERESR MA1", "Reserved", "Reserved",
    "ELD   MA1, MA0", "Reserved", "EADD  MA1, P", "ESUB  MA1, P",
    "EADD  MA1, PSH", "ESUB  MA1, PSH", "EDIVQ MA1, P", "Reserved"}
};
char XiYi[4][5] = { "X0Y0", "X0Y1", "X1Y0", "X1Y1" };
char Xi[2][3] = { "X0", "X1" };
char Yi[2][3] = { "Y0", "Y1" };
char rs[2][3] = { "ER", "ES" };
char Mi[2][4] = { "MA0", "MA1" };
char rpi[4][4] = { "RP0", "RP1", "RP2", "RP3" };
char TEC3[2][4] = { "T", "EC3" };
char rpdi[2][5] = { "RPD0", "RPD1" };
