/*  Project: Open Source Gmini Tools
    file: pack.cpp, main procedure for aospacker

    Copyright (C) 2004  The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/  
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <iostream>
#include <fstream>
  using namespace std;

#include "types.h"
#include "util.h"
#define PACKNAME "pack"

struct Buffer {
  u8 * data;
  int length;
};

int main( int argc, char **argv ) {
  char header[0x100];
  for (int i = 0; i<0x100;i++)
    header[i] = 0;
  
  if( argc != 6 ) {
    printf( "Usage: %s file.bin file.aaz loadaddress flashheader magic\n", PACKNAME );
    return -1;
  }
  Buffer bin;
  bin.length = loadFile( argv[1], &bin.data );
  ofstream aaz( argv[2], ios::out | ios::trunc );

  // building header.
  u16 chksum = chksum32( bin.data, bin.length ) & 0xffff;
  u32 loadaddress = strtol(argv[3], NULL, 16);
  u32 header_version = 2;
  char version[5] = "1000";
  u16 flashHeader;
  if (!strcmp(argv[4], "true"))
    flashHeader = 1;
  else
    flashHeader = 0;
  char magic[13];
  for (int i = 0; i< 13;i++)
    magic[i] = 0;
  strncpy(magic, argv[5], 12);

  u32 resetAddr = 0x100; 
  

  bu32( header + 0, header_version );
  bu32( header + 4, loadaddress );
  bu32( header + 8, bin.length );
  strncpy(header+12, version, 4);
  bu16( header + 16, chksum );
  bu16( header + 18, flashHeader );
  strncpy( header + 20, magic, 12 );
  bu32( header + 32, resetAddr );
  
  printf("Magic = ");
  printf("AAZ file packed with options:\n");
  printf("flashHeader = %d\n", flashHeader);
  printf("#### loadaddress 0x%x ####\n", loadaddress );

  aaz.write( header, 0x100 );
  aaz.write( ( const char * )bin.data, bin.length );
  aaz.close(  );
  delete bin.data;
  return 0;
}


